!---------------------------------------------------------------------------------------
! FLEXINVERT: convertgrid
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> convertgrid
!! 
!! Purpose:    Interpolates or averages data from a given input grid to a given 
!!             output grid 
!!
!! Interface:  
!!
!!    Inputs   
!!             midpoints -  logical (true: coords are midpoints)
!!             nx_in     -  number of grids in x dimension input grid
!!             ny_in     -  number of grids in y dimension input grid
!!             lon_in    -  longitude of input grid
!!             lat_in    -  latitude of input grid
!!             work_in   -  data on input grid
!!             nx_out    -  number of grids in x dimension output grid
!!             ny_out    -  number of grids in y dimension output grid
!!             lon_out   -  longitude of output grid
!!             lat_out   -  latitude of output grid
!!             work_out  -  data on output grid (zero matrix on input)
!!
!!    Externals
!!             sort
!!
!---------------------------------------------------------------------------------------

subroutine convertgrid(midpoints, nx_in, ny_in, lon_in, lat_in, work_in, &
                       nx_out, ny_out, lon_out, lat_out, work_out)

  use mod_var
  use mod_tools

  implicit none

  logical,                                  intent(in out) :: midpoints
  integer,                                  intent(in)     :: nx_in
  integer,                                  intent(in)     :: ny_in
  integer,                                  intent(in)     :: nx_out
  integer,                                  intent(in)     :: ny_out
  real, dimension(nx_in,ny_in,1),           intent(in out) :: work_in
  real, dimension(nx_in),                   intent(in out) :: lon_in
  real, dimension(ny_in),                   intent(in out) :: lat_in
  real, dimension(nx_out),                  intent(in out) :: lon_out
  real, dimension(ny_out),                  intent(in out) :: lat_out
  real, dimension(nx_out,ny_out,1),         intent(in out) :: work_out
  
  integer                        :: nt, ix, jy, ii, jj
  integer                        :: ix1, ix2, jy1, jy2
  real                           :: xolap1, xolap2, yolap1, yolap2
  real                           :: mass, area, aolap, aolaptot 
  real                           :: tot_out, tot_in
  integer, dimension(nx_in)      :: indx
  integer, dimension(ny_in)      :: indy
  integer                        :: ntime_in
  integer                        :: ntime_out
  real, dimension(nx_in)         :: dx_in
  real, dimension(ny_in)         :: dy_in
  real                           :: dx_out
  real                           :: dy_out
  real                           :: llx_out
  real                           :: lly_out
  real, dimension(nx_in)         :: lono
  real, dimension(ny_in)         :: lato
  real, dimension(nx_in)         :: wlon_in
  real, dimension(ny_in)         :: slat_in
  real, dimension(nx_out)        :: wlon_out
  real, dimension(ny_out)        :: slat_out
  real, dimension(nx_in,ny_in)   :: area_xy

  ntime_in = 1
  ntime_out = 1
  dx_out = dx
  dy_out = dy
  llx_out = llx
  lly_out = lly

  tot_in = 0.
  tot_out = 0.
  dx_in(:) = 0.
  dy_in(:) = 0.
  wlon_in(:) = 0.
  slat_in(:) = 0.
  wlon_out(:) = 0.
  slat_out(:) = 0.
  work_out(:,:,:) = 0.

  ! check time dimensions
  if(ntime_in.ne.ntime_out) then
    if(ntime_in.eq.1) then
      write(logid,*) 'WARNING convertgrid: repeating first time step ',ntime_out,' times'
    else
      write(logid,*) 'ERROR convertgrid: mismatch in input/output time dimension'
      stop
    endif
  endif

  ! check lon and lat dimensions
  lato = lat_in
  call sort(ny_in, lato)
  ! indices to lat
  do jy = 1, ny_in
    indy(jy) = minloc(lato,dim=1,mask=lato.eq.lat_in(jy))
  end do
  lat_in = lato
  where (lon_in.gt.180.) lon_in = lon_in - 360.
  lono = lon_in
  call sort(nx_in, lono)
  ! indices to lon
  do ix = 1, nx_in
    indx(ix) = minloc(lono,dim=1,mask=lono.eq.lon_in(ix))
  end do
  lon_in = lono
  ! reorder flx
  work_in(:,:,:) = work_in(indx,indy,:)

  ! input resolution
  do ix = 2, nx_in
    dx_in(ix) = abs(lon_in(ix)-lon_in(ix-1))
  end do
  dx_in(1) = dx_in(2)
  do jy = 2, ny_in
    dy_in(jy) = abs(lat_in(jy)-lat_in(jy-1))
  end do
  dy_in(1) = dy_in(2)

  if(midpoints) then
    ! western edges
    do ix = 1, nx_in
      wlon_in(ix) = lon_in(ix)-0.5*dx_in(ix)
    end do
    ! southern edges
    do jy = 1, ny_in
      slat_in(jy) = lat_in(jy)-0.5*dy_in(jy)
    end do
  else
    wlon_in = lon_in
    ! longitude midpoints
    do ix = 1, nx_in
      lon_in(ix) = lon_in(ix)+0.5*dx_in(ix)
    end do
    slat_in = lat_in
    ! latitude midpoints
    do jy = 1, ny_in
      lat_in(jy)=lat_in(jy)+0.5*dy_in(jy)
    end do
    midpoints = .true.
  endif  

  if( (nx_in.eq.nx_out) .and. (ny_in.eq.ny_out) ) then
    write(logid,*) 'WARNING convertgrid: input and output grids are the same'
    wlon_out = wlon_in
    slat_out = slat_in
    work_out = work_in
    go to 20
  endif

  ! loop over time and then y and x dimensions
  do nt = 1, ntime_in
    do jy = 1, ny_out
      slat_out(jy) = lly_out+float(jy-1)*dy_out
      do ix = 1, nx_out
        wlon_out(ix) = llx_out+float(ix-1)*dx_out
        ix1 = maxloc(wlon_in,dim=1,mask=wlon_in.le.wlon_out(ix))
        ix2 = maxloc(wlon_in,dim=1,mask=wlon_in.lt.(wlon_out(ix)+dx_out))
        jy1 = maxloc(slat_in,dim=1,mask=slat_in.le.slat_out(jy))
        jy2 = maxloc(slat_in,dim=1,mask=slat_in.lt.(slat_out(jy)+dy_out))
        if( (ix1.eq.0) .or. (ix2.eq.0) .or. (jy1.eq.0) .or. (jy2.eq.0) ) then
          write(logid,*) 'WARNING convertgrid: one of ix1, ix2, jy1, jy2 is zero'
          go to 10
        endif
        mass = 0.
        aolaptot = 0.
        ! loop over grid cells in input grid which overlap with the grid cell in output grid
        do jj = jy1, jy2
          do ii = ix1, ix2
            ! overlapping area has boundaries xolap1,xolap2 and yolap1,yolap2
            xolap1 = max(wlon_in(ii),wlon_out(ix))
            xolap2 = min(wlon_in(ii)+dx_in(ii),wlon_out(ix)+dx_out)
            yolap1 = max(slat_in(jj),slat_out(jy))
            yolap2 = min(slat_in(jj)+dy_in(jj),slat_out(jy)+dy_out)
            ! calculate overlapping area
            aolap = grid_area((yolap1+yolap2)/2., abs(yolap2-yolap1), abs(xolap2-xolap1))
            mass = mass + aolap*work_in(ii,jj,nt)
            aolaptot = aolaptot + aolap    
          end do
        end do
        if(aolaptot.ne.0) then
          ! area flux (kg/m2/h)
          work_out(ix,jy,nt) = mass/aolaptot
        endif
        tot_out = tot_out+mass
        ! complete righthand edge of grid
        mass = 0.
        aolaptot = 0.
        if(ix.eq.nx_out) then
          xolap1 = wlon_in(ix2)
          xolap2 = wlon_out(ix)+dx_out
          do jj = jy1, jy2
            yolap1 = max(slat_in(jj),slat_out(jy))
            yolap2 = min(slat_in(jj)+dy_in(jj),slat_out(jy)+dy_out)
            ! calculate overlapping area
            aolap = grid_area((yolap1+yolap2)/2., abs(yolap2-yolap1), abs(xolap2-xolap1))
            mass = mass + aolap*work_in(ix2,jj,nt)
            aolaptot = aolaptot + aolap
          end do
          if(aolaptot.ne.0) then
            work_out(ix,jy,nt) = mass/aolaptot
          endif   
          tot_out = tot_out + mass
        endif
        ! complete upper edge of grid
        mass = 0.
        aolaptot = 0.
        if(jy.eq.ny_out) then
          yolap1 = slat_in(jy2)
          yolap2 = slat_out(jy)+dy_out
          do ii = ix1, ix2
            xolap1 = max(wlon_in(ii),wlon_out(ix))
            xolap2 = min(wlon_in(ii)+dx_in(ii),wlon_out(ix)+dx_out)
            ! calculate overlapping area
            aolap = grid_area((yolap1+yolap2)/2., abs(yolap2-yolap1), abs(xolap2-xolap1))
            mass = mass + aolap*work_in(ii,jy2,nt)
            aolaptot = aolaptot + aolap
          end do
          if(aolaptot.ne.0) then
            work_out(ix,jy,nt) = mass/aolaptot
          endif
          tot_out = tot_out + mass
        endif
        ! check for undefined values
        if(isnan(work_out(ix,jy,nt))) work_out(ix,jy,nt) = 0.
10      continue
      end do ! nx_out
    end do ! ny_out
  end do ! ntime_in

20 continue

  ! check total in versus total out
  ix1 = maxloc(wlon_in,dim=1,mask=wlon_in.le.llx_out)
  ix2 = minloc(wlon_in,dim=1,mask=wlon_in.ge.(llx_out+nx_out*dx_out-dx_in(nx_in)))
  jy1 = maxloc(slat_in,dim=1,mask=slat_in.le.lly_out)
  jy2 = minloc(slat_in,dim=1,mask=slat_in.ge.(lly_out+ny_out*dy_out-dy_in(ny_in)))
  
  ! calculate input grid area
  area_xy(:,:) = 0.
  do jy = jy1, jy2
    do ix = ix1, ix2
      area = grid_area(lat_in(jy), dy_in(jy), dx_in(ix))
      area_xy(ix,jy) = area
    end do
  end do
  
  ! calculate total in
  do nt = 1, ntime_in
    tot_in = tot_in + sum( sum(work_in(:,:,nt)*area_xy(:,:),dim=1) )
  end do

  write(logid,fmt='(1x,''Total mass in:'',1x,E12.4,1x,''Tg/y'')') tot_in*24.*365./float(ntime_in)/1.e9
  write(logid,fmt='(1x,''Total mass out:'',1x,E12.4,1x,''Tg/y'')') tot_out*24.*365./float(ntime_in)/1.e9

  do ix = 1, nx_out
    lon_out(ix) = wlon_out(ix)+dx_out*0.5
  end do
  do jy = 1, ny_out
    lat_out(jy) = slat_out(jy)+dy_out*0.5
  end do

  if( (ntime_in.eq.1).and.(ntime_out.gt.1)) then
    do nt = 2, ntime_out 
      work_out(:,:,nt) = work_out(:,:,1)
    end do
  endif


end subroutine convertgrid


