!---------------------------------------------------------------------------------------
! FLEXINVERT: read_ncdf
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> read_ncdf
!!
!! Purpose:    Reads a ncdf file with a 3D floating variable.
!!
!! Interface:
!!
!!    Inputs
!!             filename  -  name of ncdf file
!!             varname   -  variable name to read
!!             lonname   -  name of longitude dimension variable
!!             latname   -  name of latitude dimension variable
!!             timename  -  name of time dimension variable
!!             lono      -  origin on longitudinal dimension
!!             lato      -  origin on latitudinal dimension
!!             nx        -  number of grid cells in longitudinal dimension
!!             ny        -  number of grid cells in latitudinal dimension
!!             jd        -  origin in julian days 
!!             nread     -  number of fields to read 
!!             cnt       -  actual number of fields read (</= nread)
!!             flx       -  flux variable (zero array on input)
!!
!!    Externals
!!             check
!!
!---------------------------------------------------------------------------------------

subroutine read_ncdf(filename, varname, lonname, latname, timename,&
                     lono, lato, nx, ny, jd, nread, cnt, flx)

  use mod_var
  use mod_dates 
  use mod_save,    only : check
  use netcdf

  implicit none

  character(len=max_path_len),    intent(in)     :: filename
  character(len=max_name_len),    intent(in)     :: varname, lonname, latname, timename
  real,                           intent(in)     :: lono, lato
  integer,                        intent(in)     :: nx, ny, nread
  integer,                        intent(in out) :: cnt
  real(kind=8),                   intent(in)     :: jd
  real, dimension(nx,ny,nread),   intent(in out) :: flx

  integer                                        :: ncid, lon_dimid, lat_dimid, time_dimid 
  integer                                        :: ix, jy, kt
  integer                                        :: lonid, latid, tid, varid
  integer                                        :: nlon, nlat, nt
  real(kind=8)                                   :: jdi
  real                                           :: xres, yres
  real, dimension(:), allocatable                :: lon
  real, dimension(:), allocatable                :: lat
  real, dimension(:), allocatable                :: time

  ! adjust to days since 1-Jan-1900
  jdi = jd - juldate(19000101,0)

  call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
  call check( nf90_inq_dimid(ncid,trim(lonname),lon_dimid) )
  call check( nf90_inq_dimid(ncid,trim(latname),lat_dimid) )
  call check( nf90_inq_dimid(ncid,trim(timename),time_dimid) )

  call check( nf90_inquire_dimension(ncid,lon_dimid,len=nlon) )
  call check( nf90_inquire_dimension(ncid,lat_dimid,len=nlat) )
  call check( nf90_inquire_dimension(ncid,time_dimid,len=nt) )

  allocate( lon(nlon) )
  allocate( lat(nlat) )
  allocate( time(nt) )

  call check( nf90_inq_varid(ncid,trim(lonname),lonid) )
  call check( nf90_get_var(ncid,lonid,lon) )
  call check( nf90_inq_varid(ncid,trim(latname),latid) )
  call check( nf90_get_var(ncid,latid,lat) )
  call check( nf90_inq_varid(ncid,trim(timename),tid) )
  call check( nf90_get_var(ncid,tid,time) )

  xres = lon(2) - lon(1)
  yres = lat(2) - lat(1)

  write(logid,*) 'read_ncdf: lono, lato = ',lono,lato
  write(logid,*) 'read_ncdf: xres, yres = ',xres,yres 
  write(logid,*) 'read_ncdf: lon(1), lat(1) = ',lon(1),lat(1) 

  if ( ((lon(1)-xres/2.).gt.lono).or.((lat(1)-yres/2.).gt.lato) ) then
    write(logid,*) 'ERROR: incompatible dimensions in '//trim(filename)
    stop
  endif

  ! check dimension ranges
  ! note: lono and lato are the western and southern edges
  if( (lono.lt.minval(lon-xres/2.)).or.(lono.gt.maxval(lon-xres/2.)) ) then
    write(logid,*) 'ERROR: longitude out of range in '//trim(filename)
    stop
  endif
  if( (lato.lt.minval(lat-yres/2.)).or.(lato.gt.maxval(lat-yres/2.)) ) then
    write(logid,*) 'ERROR: latitude out of range in '//trim(filename)
    stop
  endif

  ! indices to variable
  ix = minloc(lon, dim=1, mask=abs(lon-xres/2.-lono).eq.minval(abs(lon-xres/2.-lono)))
  jy = minloc(lat, dim=1, mask=abs(lat-yres/2.-lato).eq.minval(abs(lat-yres/2.-lato)))
  kt = minloc(time, dim=1, mask=abs(time-jdi).eq.minval(abs(time-jdi)))
  write(logid,*) 'read_ncdf: ix, jy, kt = ',ix,jy,kt

  ! check length to read 
  if( (kt+nread-1).gt.nt ) then
    write(logid,*) 'WARNING: requested number of fields to read outside range'
    write(logid,*) 'reading until the end of the file'
    cnt = nt - kt + 1
  else
    cnt = nread
  endif

  call check( nf90_inq_varid(ncid,trim(varname),varid) )
  call check( nf90_get_var(ncid,varid,flx(:,:,1:cnt),start=(/ix,jy,kt/),count=(/nx,ny,cnt/)) )

  call check( nf90_close(ncid) )

  deallocate(lon)
  deallocate(lat)
  deallocate(time)

end subroutine read_ncdf

